# -*- coding: utf-8 -*-

# Copyright (c) 2015-2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

"""Add theme to FullTextSearch

Revision ID: 53ba1a68d5fe
Revises: 5109242131ce
Create Date: 2015-08-05 14:43:30.889188
"""

from alembic import op, context
from sqlalchemy import Column, Integer, ForeignKey, Boolean, String
from c2cgeoportal.lib.sqlalchemy_ import JSONEncodedDict

# revision identifiers, used by Alembic.
revision = '53ba1a68d5fe'
down_revision = '5109242131ce'


def upgrade():
    schema = context.get_context().config.get_main_option('schema')

    op.add_column('tsearch', Column(
        'interface_id', Integer,
        ForeignKey(schema + '.interface.id'),
        nullable=True
    ), schema=schema)
    op.add_column('tsearch', Column('lang', String(2), nullable=True), schema=schema)
    op.add_column('tsearch', Column('actions', JSONEncodedDict, nullable=True), schema=schema)
    op.add_column('tsearch', Column('from_theme', Boolean, server_default='false'), schema=schema)

    op.create_index(
        'tsearch_search_index',
        table_name='tsearch',
        columns=['ts', 'public', 'role_id', 'interface_id', 'lang'],
        schema=schema
    )


def downgrade():
    schema = context.get_context().config.get_main_option('schema')

    op.drop_index('tsearch_search_index', schema=schema)

    op.drop_column('tsearch', 'interface_id', schema=schema)
    op.drop_column('tsearch', 'lang', schema=schema)
    op.drop_column('tsearch', 'actions', schema=schema)
    op.drop_column('tsearch', 'from_theme', schema=schema)



# -*- coding: utf-8 -*-

# Copyright (c) 2014-2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


import sys
import transaction
from json import loads
from argparse import ArgumentParser
from pyramid.paster import get_app


def main():
    parser = ArgumentParser(
        prog=sys.argv[0], add_help=True,
        description="Tool used to migrate your old layers from the old structure to the new one.",
    )

    parser.add_argument(
        "-i", "--app-config",
        default="production.ini",
        dest="app_config",
        help="the application .ini config file (optional, default is 'production.ini')"
    )
    parser.add_argument(
        "-n", "--app-name",
        default="app",
        dest="app_name",
        help="the application name (optional, default is 'app')"
    )
    parser.add_argument(
        "--no-layers",
        dest="layers",
        action="store_false",
        help="don't import the layers"
    )
    parser.add_argument(
        "--no-groups",
        dest="groups",
        action="store_false",
        help="don't import the groups"
    )
    options = parser.parse_args()

    app_config = options.app_config
    app_name = options.app_name
    if app_name is None and "#" in app_config:
        app_config, app_name = app_config.split("#", 1)
    get_app(app_config, name=app_name)

    # must be done only once we have loaded the project config
    from c2cgeoportal.models import DBSession, \
        ServerOGC, LayerWMS, LayerWMTS, LayerV1, LayerGroup

    session = DBSession()

    if options.layers:
        table_list = [LayerWMTS, LayerWMS, ServerOGC]
        for table in table_list:
            print("Emptying table %s." % str(table.__table__))
            # must be done exactly this way othewise the cascade config in the
            # models are not used
            for t in session.query(table).all():
                session.delete(t)

        # list and create all distinct server_ogc
        server_ogc(session)

        print("Converting layerv1.")
        for layer in session.query(LayerV1).all():
            layer_v1tov2(session, layer)

    if options.groups:
        print("Converting layer group.")
        for group in session.query(LayerGroup).all():
            layergroup_v1tov2(session, group)

    transaction.commit()


def server_ogc(session):
    from c2cgeoportal.models import LayerV1, ServerOGC

    servers_v1 = session.query(
        LayerV1.url, LayerV1.image_type, LayerV1.is_single_tile).group_by(
            LayerV1.url, LayerV1.image_type, LayerV1.is_single_tile).all()
    unique_servers = []

    # get existing list of server_ogc
    servers_ogc = session.query(ServerOGC).all()
    for server in servers_ogc:
        identifier = str(server.url) + ' ' + str(server.image_type) + ' ' + \
            str(server.is_single_tile)
        if identifier not in unique_servers:
            unique_servers.append(identifier)

    # add new server_ogc
    for server in servers_v1:
        # default image_type
        image_type = server[1]
        if server[1] is None:
            image_type = 'image/png'
        identifier = str(server[0]) + ' ' + image_type + ' ' + str(server[2])
        if identifier not in unique_servers:
            unique_servers.append(identifier)
            new_server_ogc = ServerOGC()
            new_server_ogc.url = server[0]
            new_server_ogc.image_type = image_type
            new_server_ogc.is_single_tile = server[2]
            name = server[0]
            if name is None:
                name = str(server[1])
            if server[2]:
                name += ' with single_tile'
            new_server_ogc.name = u'source for %s' % name

            session.add(new_server_ogc)

    transaction.commit()


def layer_v1tov2(session, layer):
    from c2cgeoportal.models import ServerOGC, LayerWMS, LayerWMTS, \
        LayergroupTreeitem, WMTSDimension

    if layer.layer_type == "internal WMS" or layer.layer_type == "external WMS":
        # use the first one
        new_layer = LayerWMS()
        image_type = layer.image_type
        if layer.image_type is None:
            image_type = 'image/png'
        server_ogc = session.query(ServerOGC).filter(
            ServerOGC.url == layer.url,
            ServerOGC.image_type == image_type, ServerOGC.is_single_tile ==
            layer.is_single_tile).one()
        new_layer.server_ogc = server_ogc
    elif layer.layer_type == "WMTS":
        new_layer = LayerWMTS()

    new_layer.name = layer.name
    new_layer.public = layer.public
    new_layer.geo_table = layer.geo_table
    new_layer.interfaces = layer.interfaces

    for link in layer.parents_relation:
        new_link = LayergroupTreeitem()
        new_link.ordering = link.ordering
        new_link.treegroup_id = link.treegroup_id
        new_link.group = link.group
        new_link.item = new_layer

    if layer.layer_type[-4:] == " WMS":
        new_layer.layer = layer.name
        new_layer.style = layer.style
        new_layer.time_mode = layer.time_mode
        new_layer.time_widget = layer.time_widget

    if layer.layer_type == "WMTS":
        new_layer.url = layer.url
        new_layer.layer = layer.name
        new_layer.style = layer.style
        new_layer.matrix_set = layer.matrix_set
        new_layer.image_type = layer.image_type

        if layer.dimensions is not None:
            dimensions = loads(layer.dimensions)
            for name, value in dimensions.items():
                session.add(WMTSDimension(name, value, new_layer))

    layer_add_ui_metadata(layer, new_layer, session)

    session.add(new_layer)


def new_uimetadata(name, value, item):
    from c2cgeoportal.models import UIMetadata

    uimetadata = UIMetadata(name, value)
    uimetadata.item = item
    return uimetadata


def layer_add_ui_metadata(layer, new_layer, session):
    if layer.metadata_url is not None:
        session.add(new_uimetadata(u"metadataUrl", layer.metadata_url, new_layer))
    if layer.is_checked is True:
        session.add(new_uimetadata(u"isChecked", u"true", new_layer))
    if layer.icon is not None:
        session.add(new_uimetadata(u"icon", layer.icon, new_layer))
    if layer.wms_url is not None:
        session.add(new_uimetadata(u"wmsUrl", layer.wms_url, new_layer))
    if layer.wms_layers is not None:
        session.add(new_uimetadata(u"wmsLayers", layer.wms_layers, new_layer))
    if layer.query_layers is not None:
        session.add(new_uimetadata(u"queryLayers", layer.query_layers, new_layer))
    if layer.legend is not None:
        session.add(new_uimetadata(u"legend", layer.legend, new_layer))
    if layer.legend_image is not None:
        session.add(new_uimetadata(u"legendImage", layer.legend_image, new_layer))
    if layer.legend_rule is not None:
        session.add(new_uimetadata(u"legendRule", layer.legend_rule, new_layer))
    if layer.is_legend_expanded is True:
        session.add(new_uimetadata(u"isLegendExpanded", u"true", new_layer))
    if layer.min_resolution is not None:
        session.add(new_uimetadata(u"minResolution", layer.min_resolution, new_layer))
    if layer.max_resolution is not None:
        session.add(new_uimetadata(u"maxResolution", layer.max_resolution, new_layer))
    if layer.disclaimer is not None:
        session.add(new_uimetadata(u"disclaimer", layer.disclaimer, new_layer))
    if layer.identifier_attribute_field is not None:
        session.add(new_uimetadata(
            u"identifier_attribute_field",
            layer.identifier_attribute_field, new_layer
        ))
    if layer.exclude_properties is not None:
        session.add(new_uimetadata("excludeProperties", layer.exclude_properties, new_layer))


def layergroup_v1tov2(session, group):
    is_expended_metadatas = group.get_metadatas("isExpanded")
    if group.is_expanded is True:
        if len(is_expended_metadatas) > 0:
            is_expended_metadatas[0].value = u"true"
        else:
            session.add(new_uimetadata(u"isExpanded", u"true", group))
    elif len(is_expended_metadatas) > 0:
        session.delete(is_expended_metadatas)




# Copyright (c) 2015-2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


"""Move user table to static schema

Revision ID: 1da396a88908
Revises: 3f89a7d71a5e
Create Date: 2015-02-20 14:09:04.875390
"""

from alembic import op, context
from sqlalchemy import Column, ForeignKey
from sqlalchemy.types import Integer, String, Unicode, Boolean

# revision identifiers, used by Alembic.
revision = '1da396a88908'
down_revision = '3f89a7d71a5e'


def upgrade():
    schema = context.get_context().config.get_main_option('schema')
    staticschema = schema + '_static'
    parentschema = context.get_context().config.get_main_option('parentschema')

    engine = op.get_bind().engine
    if type(engine).__name__ != 'MockConnection' and \
            op.get_context().dialect.has_table(
                engine, 'user', schema=staticschema):  # pragma: nocover
        return

    op.create_table(
        'user',
        Column('type', String(10), nullable=False),
        Column('id', Integer, primary_key=True),
        Column('username', Unicode, unique=True, nullable=False),
        Column('password', Unicode, nullable=False),
        Column('email', Unicode, nullable=False),
        Column('is_password_changed', Boolean, default=False),
        Column('role_name', String),
        schema=staticschema,
    )
    parent_column = ''
    parent_select = ''
    parent_join = ''
    if parentschema is not None and parentschema is not '':  # pragma: nocover
        op.add_column(
            'user',
            Column('parent_role_name', String),
            schema=staticschema
        )
        parent_column = ', parent_role_name'
        parent_select = ', pr.name'
        parent_join = (
            'LEFT OUTER JOIN %(parentschema)s.role AS pr ON (pr.id = u.parent_role_id)' % {
                'parentschema': parentschema,
            }
        )

    op.execute(
        'INSERT INTO %(staticschema)s.user '
        '(type, username, password, email, is_password_changed, role_name%(parent_column)s) ('
        'SELECT u.type, u.username, u.password, u.email, '
        'u.is_password_changed, r.name%(parent_select)s '
        'FROM %(schema)s.user AS u '
        'LEFT OUTER JOIN %(schema)s.role AS r ON (r.id = u.role_id) %(parent_join)s'
        ')' % {
            'staticschema': staticschema,
            'schema': schema,
            'parent_select': parent_select,
            'parent_column': parent_column,
            'parent_join': parent_join,
        }
    )

    op.drop_table('user', schema=schema)


def downgrade():
    schema = context.get_context().config.get_main_option('schema')
    staticschema = schema + '_static'
    parentschema = context.get_context().config.get_main_option('parentschema')

    op.create_table(
        'user',
        Column('type', String(10), nullable=False),
        Column('id', Integer, primary_key=True),
        Column('username', Unicode, unique=True, nullable=False),
        Column('password', Unicode, nullable=False),
        Column('email', Unicode, nullable=False),
        Column('is_password_changed', Boolean, default=False),
        Column('role_id', Integer, ForeignKey(schema + '.role.id'), nullable=False),
        schema=schema,
    )
    parent_column = ''
    parent_select = ''
    parent_join = ''
    if parentschema is not None and parentschema is not '':  # pragma: nocover
        op.add_column(
            'user',
            Column('parent_role_id', Integer, ForeignKey(parentschema + '.role.id')),
            schema=schema
        )
        parent_column = ', parent_role_id'
        parent_select = ', pr.id'
        parent_join = (
            'LEFT OUTER JOIN %(parentschema)s.role AS pr ON (pr.name = u.parent_role_name)' % {
                'parentschema': parentschema,
            }
        )

    op.execute(
        'INSERT INTO %(schema)s.user '
        '(type, username, password, email, is_password_changed, role_id%(parent_column)s) ('
        'SELECT u.type, u.username, u.password, u.email, '
        'u.is_password_changed, r.id%(parent_select)s '
        'FROM %(staticschema)s.user AS u '
        'LEFT OUTER JOIN %(schema)s.role AS r ON (r.name = u.role_name) %(parent_join)s'
        ')' % {
            'staticschema': staticschema,
            'schema': schema,
            'parent_select': parent_select,
            'parent_column': parent_column,
            'parent_join': parent_join,
        }
    )

    op.drop_table('user', schema=staticschema)



# -*- coding: utf-8 -*-

# Copyright (c) 2011-2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


import os
import re
import subprocess
from json import loads, dumps
from urlparse import parse_qsl
from argparse import ArgumentParser


def _sub(pattern, repl, string, count=0, flags=0):
    new_string = re.sub(pattern, repl, string, count=count, flags=flags)
    if new_string == string:
        print("Unable to find the pattern:")
        print(pattern)
        print("in")
        print(string)
        exit(1)
    return new_string


def _subs(subs, string):
    for pattern, repl in subs:
        if repl is False:
            if re.search(pattern, string) is not None:
                return string
        else:
            new_string = re.sub(pattern, repl, string)
            if new_string != string:
                return new_string
    print("Unable to find the patterns:")
    print("\n".join([p for p, r in subs]))
    print("in")
    print(string)
    exit(1)


class _RouteDest:

    def __init__(self, constant, route):
        self.constant = constant
        self.route = route

    def __call__(self, matches):
        query_string = matches.group(1)
        query = ''
        if len(query_string) > 0:
            query = ", _query=%s" % dumps(dict(parse_qsl(query_string)))
        return r"module.constant('%s', '${request.route_url('%s'%s) | n}');" % (
            self.constant, self.route, query
        )

RE_NPM_VERSION = re.compile("^([0-9]+)\.([0-9]+)\.([0-9]+)$")
RE_NPM_PRERELEASE_VERSION = re.compile("^([0-9]+)\.([0-9]+)\.([0-9]+)\.?([a-z]+)([0-9]+)$")


def _ngeo_version():
    if os.environ["TRAVIS_TAG"]:
        match = RE_NPM_VERSION.match(os.environ["TRAVIS_TAG"])
        prerelease_match = RE_NPM_PRERELEASE_VERSION.match(os.environ["TRAVIS_TAG"])
        if match is not None:
            return "{}.{}.{}".format(match.group(1), match.group(2), match.group(3))
        if prerelease_match is not None:
            return "{}.{}.{}-{}.{}".format(
                prerelease_match.group(1),
                prerelease_match.group(2),
                prerelease_match.group(3),
                prerelease_match.group(4),
                prerelease_match.group(5)
            )
    return None


def _ngeo_git_version():
    version = _ngeo_version()
    if version is not None:
        return version
    if os.environ["TRAVIS_TAG"]:
        return os.environ["TRAVIS_TAG"]
    return "master"


def ngeo_git_version():
    print(_ngeo_git_version())


def _get_ngeo_version():
    version = _ngeo_version()
    if version is not None:
        return version
    return "git://github.com/camptocamp/ngeo#{}".format(
        subprocess.check_output(["git", "rev-parse", "HEAD"], cwd="ngeo").strip()
    )


def main():
    """
    Import the ngeo apps files
    """

    parser = ArgumentParser(description='import ngeo apps files')

    parser.add_argument('--html', action="store_true", help="Import the html template")
    parser.add_argument('--js', action="store_true", help="Import the javascript controller")
    parser.add_argument('--package', action="store_true", help="Import the package JSON")
    parser.add_argument('interface', metavar='INTERFACE', help="The interface we import")
    parser.add_argument('src', metavar='SRC', help="The ngeo source file")
    parser.add_argument('dst', metavar='DST', help="The destination file")

    args = parser.parse_args()

    with open(args.src) as src:
        data = src.read()

        if args.package:
            ngeo_json_data = loads(data)
            json_data = {}
            json_data["name"] = "{{package}}"
            json_data["version"] = "2.0.0"
            json_data["description"] = "A GeoMapFish project"

            json_data["devDependencies"] = ngeo_json_data["devDependencies"]
            # freeze the ngeo version
            json_data["devDependencies"]["ngeo"] = _get_ngeo_version()
            for package in [
                "angular-jsdoc",
                "angular-mocks",
                "coveralls",
                "fs-extra",
                "gaze",
                "graceful-fs",
                "jsdoc",
                "jsdom",
                "karma",
                "karma-coverage",
                "karma-jasmine",
                "karma-phantomjs-launcher",
                "nomnom",
                "temp",
                "walk",
            ]:
                del json_data["devDependencies"][package]

            data = dumps(json_data, indent=4, sort_keys=True)
            data = _sub(r" +\n", "\n", data)

        else:
            data = re.sub(r"{{", r"\\{\\{", data)
            data = re.sub(r"}}", r"\\}\\}", data)
            data = re.sub("app", "{{package}}", data)

# temporary disable ...
#        if args.js:
            # Full text search
#            data = _sub(r"datasetTitle: 'Internal',", r"datasetTitle: '{{project}}',", data)

        if args.html:
            data = "<%\n" \
                "from json import dumps\n" \
                "%>\n" + \
                data
            # back for ng-app
            data = _sub(r"ng-{{package}}", r"ng-app", data)
            if args.interface == "mobile":
                # back for mobile-web-app-capable
                data = _sub(
                    r"mobile-web-{{package}}-capable",
                    r"mobile-web-app-capable", data
                )
            # Styles
            data = _sub(
                r'    <link rel="stylesheet.*/build/{}.css">'.format(args.interface),
                r"""% if debug:
    <link rel="stylesheet/less" href="${{request.static_url('%s/ngeo/contribs/gmf/less/font.less' % request.registry.settings['node_modules_path'])}}" type="text/css">
    <link rel="stylesheet/less" href="${{request.static_url('{{{{package}}}}:static-ngeo/less/{interface}.less')}}" type="text/css">
    <link rel="stylesheet/less" href="${{request.static_url('%s/ngeo/contribs/gmf/less/{interface}.less' % request.registry.settings['node_modules_path'])}}" type="text/css">
% else:
    <link rel="stylesheet" href="${{request.static_url('{{{{package}}}}:static-ngeo/build/{interface}.css')}}" type="text/css">
% endif""".format(interface=args.interface),  # noqa
                data,
                count=1,
                flags=re.DOTALL
            )
            # Scripts
            data = _sub(
                r'    <script .*watchwatchers.js"></script>',
                r"""% if debug:
    <script>
        window.CLOSURE_BASE_PATH = '';
        window.CLOSURE_NO_DEPS = true;
    </script>
    <script src="${{request.static_url('%s/jquery/dist/jquery.js' % request.registry.settings['node_modules_path'])}}"></script>
    <script src="${{request.static_url('%s/angular/angular.js' % request.registry.settings['node_modules_path'])}}"></script>
    <script src="${{request.static_url('%s/angular-gettext/dist/angular-gettext.js' % request.registry.settings['node_modules_path'])}}"></script>
    <script src="${{request.static_url('%s/angular-animate/angular-animate.js' % request.registry.settings['node_modules_path'])}}"></script>
    <script src="${{request.static_url('%s/angular-touch/angular-touch.js' % request.registry.settings['node_modules_path'])}}"></script>
    <script src="${{request.static_url('%s/bootstrap/dist/js/bootstrap.js' % request.registry.settings['node_modules_path'])}}"></script>
    <script src="${{request.static_url('%s/proj4/dist/proj4-src.js' % request.registry.settings['node_modules_path'])}}"></script>
    <script src="${{request.static_url('%s/d3/d3.min.js' % request.registry.settings['node_modules_path'])}}"></script>
    <script src="${{request.static_url('%s/typeahead.js/dist/typeahead.bundle.js' % request.registry.settings['node_modules_path'])}}"></script>
    <script src="${{request.static_url('%s/closure/goog/base.js' % request.registry.settings['closure_library_path'])}}"></script>
    <script src="${{request.route_url('deps.js')}}"></script>
    <script>
        goog.require('{{{{package}}}}_{interface}');
    </script>
    <script src="${{request.static_url('{{{{package}}}}:static-ngeo/build/templatecache.js')}}"></script>
    <script src="${{request.static_url('%s/ngeo/utils/watchwatchers.js' % request.registry.settings['node_modules_path'])}}"></script>
    <script src="${{request.static_url('%s/less/dist/less.min.js' % request.registry.settings['node_modules_path'])}}"></script>
% else:
    <script src="${{request.static_url('{{{{package}}}}:static-ngeo/build/{interface}.js')}}"></script>
% endif""".format(interface=args.interface),  # noqa
                data,
                count=1,
                flags=re.DOTALL
            )
            # i18n
            data = _sub(
                "module.constant\('defaultLang', 'en'\);",
                "module.constant('defaultLang', "
                "'${request.registry.settings[\"default_locale_name\"]}');",
                data,
            )
            data = _sub(
                "module.constant\('langUrls', {[^}]*}\);",
                r"""module.constant('langUrls', {
${ ',\\n'.join([
    "             '{lang}': '{url}'".format(
        lang=lang,
        url=request.static_url('{{package}}:static-ngeo/build/{lang}.json'.format(lang=lang))
    )
    for lang in request.registry.settings["available_locale_names"]
]) | n}
           });""",
                data,
            )
            data = _subs(
                [(
                    "module.constant\('gmfSearchGroups', \[\]\);",
                    False
                ), (
                    "module.constant\('gmfSearchGroups', \[[^\]]*\]\);",
                    "module.constant('gmfSearchGroups', ${dumps(fulltextsearch_groups) | n});",
                )],
                data,
            )

            # replace routes
            for constant, url_end, route in [
                ("authenticationBaseUrl", r"", "base"),
                ("fulltextsearchUrl", r"/fulltextsearch", "fulltextsearch"),
                ("gmfWmsUrl", r"/mapserv_proxy", "mapserverproxy"),
                ("gmfTreeUrl", r"/themes", "themes"),
            ]:
                data = _sub(
                    r"module.constant\('%s', "
                    "'https://geomapfish-demo.camptocamp.net/2.0/wsgi%s\??([^\']*)'\);" % (
                        constant, url_end
                    ),
                    _RouteDest(constant, route),
                    data,
                )

        with open(args.dst, "wt") as dst:
            dst.write(data)



# Copyright (c) 2016, Camptocamp SA
# All rights reserved.

# Auto-generated file. Do not Edit!
INFO = {
    "git_hash": "bba89f318a678c01900b21ed12123bbdfb0d10b5",
    "version": "2.0",
    "git_tag": "2.1.0.dev4",
}



# -*- coding: utf-8 -*-

# Copyright (c) 2011-2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


import logging

from pyramid.view import view_config

from c2cgeoportal.lib.caching import get_region, NO_CACHE, PUBLIC_CACHE, PRIVATE_CACHE
from c2cgeoportal.lib.functionality import get_mapserver_substitution_params
from c2cgeoportal.lib.filter_capabilities import filter_capabilities
from c2cgeoportal.views.proxy import Proxy

cache_region = get_region()
log = logging.getLogger(__name__)


class MapservProxy(Proxy):

    def __init__(self, request):
        Proxy.__init__(self, request)
        self.settings = request.registry.settings.get("mapserverproxy", {})

    def _get_wms_url(self):
        return self.settings.get("external_mapserv_url") if \
            self.external else \
            self.settings.get("mapserv_url")

    def _get_wfs_url(self):
        internal_url = self.settings.get(
            "mapserv_wfs_url",
            self.settings.get("mapserv_url")
        )
        external_url = self.settings.get(
            "external_mapserv_wfs_url",
            self.settings.get("external_mapserv_url")
        )
        return external_url if self.external else internal_url

    @view_config(route_name="mapserverproxy")
    def proxy(self):

        self.user = self.request.user
        self.external = bool(self.request.params.get("EXTERNAL", None))

        # params hold the parameters we"re going to send to MapServer
        params = dict(self.request.params)

        # reset possible value of role_id and user_id
        if "role_id" in params:  # pragma: no cover
            del params["role_id"]
        if "user_id" in params:  # pragma: no cover
            del params["user_id"]

        self.lower_params = self._get_lower_params(params)

        if self.user is not None and not self.settings["geoserver"]:
            # We have a user logged in. We need to set group_id and
            # possible layer_name in the params. We set layer_name
            # when either QUERY_PARAMS or LAYERS is set in the
            # WMS params, i.e. for GetMap and GetFeatureInfo
            # requests. For GetLegendGraphic requests we don't
            # send layer_name, but MapServer shouldn't use the DATA
            # string for GetLegendGraphic.

            params["role_id"] = self.user.parent_role.id if self.external else self.user.role.id

            # In some application we want to display the features owned by a user
            # than we need his id.
            if not self.external:
                params["user_id"] = self.user.id  # pragma: nocover

        # don't allows direct variable substitution
        for k in params.keys():
            if k[:2].capitalize() == "S_":
                log.warning("Direct substitution not allowed (%s=%s)." %
                            (k, params[k]))
                del params[k]

        # add functionalities params
        params.update(get_mapserver_substitution_params(self.request))

        # get method
        method = self.request.method

        # we want the browser to cache GetLegendGraphic and
        # DescribeFeatureType requests
        use_cache = False

        if method == "GET":
            # For GET requests, params are added only if the self.request
            # parameter is actually provided.
            if "request" not in self.lower_params:
                params = {}  # pragma: no cover
            else:
                use_cache = self.lower_params["request"] in (
                    u"getcapabilities",
                    u"getlegendgraphic",
                    u"describelayer",
                    u"describefeaturetype",
                )

                # no user_id and role_id or cached queries
                if use_cache and "user_id" in params:
                    del params["user_id"]
                if use_cache and "role_id" in params:
                    del params["role_id"]

            if "service" in self.lower_params and self.lower_params["service"] == u"wfs":
                _url = self._get_wfs_url()
            else:
                _url = self._get_wms_url()
        else:
            # POST means WFS
            _url = self._get_wfs_url()

        cache_control = PRIVATE_CACHE
        if method == "GET" and \
                "service" in self.lower_params and \
                self.lower_params["service"] == u"wms":
            if self.lower_params["request"] in (u"getmap", u"getfeatureinfo"):
                cache_control = NO_CACHE
            elif self.lower_params["request"] == u"getlegendgraphic":
                cache_control = PUBLIC_CACHE
        elif method == "GET" and \
                "service" in self.lower_params and \
                self.lower_params["service"] == u"wfs":
            if self.lower_params["request"] == u"getfeature":
                cache_control = NO_CACHE
        elif method != "GET":
            cache_control = NO_CACHE

        role = None if self.user is None else \
            self.user.parent_role if self.external else self.user.role

        headers = self._get_headers()
        # Add headers for Geoserver
        if self.settings["geoserver"] and self.user is not None:
            headers["sec-username"] = self.user.username
            headers["sec-roles"] = role.name

        response = self._proxy_callback(
            role.id if role is not None else None, cache_control,
            url=_url, params=params, cache=use_cache,
            headers=headers, body=self.request.body
        )
        return response

    def _proxy_callback(self, role_id, cache_control, *args, **kwargs):
        params = kwargs.get("params", {})
        callback = params.get("callback", None)
        if callback is not None:
            del params["callback"]
        resp, content = self._proxy(*args, **kwargs)

        if self.lower_params.get("request") == "getcapabilities":
            content = filter_capabilities(
                content, role_id, self.lower_params.get("service") == "wms",
                self._get_wms_url(),
                self.request.headers,
                self.settings.get("proxies", None)
            )

        content_type = None
        if callback is not None:
            content_type = "application/javascript"
            # escape single quotes in the JavaScript string
            content = unicode(content.decode("utf8"))
            content = content.replace(u"'", u"\\'")
            content = u"%s('%s');" % (callback, u" ".join(content.splitlines()))
        else:
            content_type = resp["content-type"]

        return self._build_response(
            resp, content, cache_control, "mapserver",
            content_type=content_type
        )



# -*- coding: utf-8 -*-

# Copyright (c) 2011-2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


from pyramid.i18n import get_localizer, TranslationStringFactory
from pyramid.events import subscriber, BeforeRender, NewRequest


@subscriber(BeforeRender)
def add_renderer_globals(event):
    request = event.get("request")
    if request:
        event["_"] = request.translate
        event["localizer"] = request.localizer

tsf = TranslationStringFactory("c2cgeoportal")


@subscriber(NewRequest)
def add_localizer(event):
    request = event.request
    localizer = get_localizer(request)

    def auto_translate(string):
        return localizer.translate(tsf(string))

    request.localizer = localizer
    request.translate = auto_translate



# -*- coding: utf-8 -*-

# Copyright (c) 2011-2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


import os
import urllib
import httplib
import logging
from httplib2 import Http
from json import dumps, loads
from time import sleep
from urlparse import urlsplit, urlunsplit
from subprocess import check_output, CalledProcessError

from pyramid.view import view_config
from pyramid.response import Response

from c2cgeoportal.lib import add_url_params

log = logging.getLogger(__name__)


def build_url(name, url, request, headers=None):
    if headers is None:
        headers = {}
    headers["Cache-Control"] = "no-cache"

    urlfragments = urlsplit(url)
    if urlfragments.netloc == request.environ.get("SERVER_NAME"):
        url_ = urlunsplit((
            "http", "localhost", urlfragments.path, urlfragments.query, urlfragments.fragment
        ))
        headers["Host"] = urlfragments.netloc
    else:
        url_ = url

    settings = request.registry.settings.get("checker", {})
    for header in settings.get("forward_headers", []):
        value = request.headers.get(header)
        if value is not None:
            headers[header] = value

    log.info("%s, URL: %s => %s" % (name, url, url_))
    return url_, headers


class Checker(object):  # pragma: no cover

    status_int = httplib.OK
    status = httplib.responses[httplib.OK]

    def __init__(self, request):
        self.request = request
        self.settings = self.request.registry.settings["checker"].get("defaults", {})
        type_ = request.params.get("type")
        if type_ is not None:
            self.settings.update(self.request.registry.settings["checker"].get(type_, {}))

    def set_status(self, code, text):
        if int(code) >= self.status_int:
            self.status_int = int(code)
            self.status = text

    def make_response(self, msg):
        return Response(
            body=msg, status="%i %s" % (self.status_int, self.status), cache_control="no-cache"
        )

    def testurl(self, url):
        url, headers = build_url("Check", url, self.request)

        h = Http()
        resp, content = h.request(url, headers=headers)

        if resp.status != httplib.OK:
            print(resp.items())
            self.set_status(resp.status, resp.reason)
            return url + "<br/>" + content

        return "OK"

    def testurls(self, named_urls):
        results = [
            "{}: {}".format(name, self.testurl(url))
            for name, url in named_urls
        ]
        return self.make_response(
            "OK" if len(results) == 0 else "\n\n".join(results)
        )

    @view_config(route_name="checker_routes")
    def routes(self):
        named_urls = [
            (
                route["name"],
                add_url_params(
                    self.request.route_url(route["name"]),
                    route.get("params", {}),
                ),
            )
            for route in self.settings["routes"]
            if route["name"] not in self.settings["routes_disable"]
        ]
        return self.testurls(named_urls)

    @view_config(route_name="checker_pdf3")
    def pdf3(self):
        return self.make_response(self._pdf3())

    def _pdf3(self):
        body = dumps(self.settings["print_spec"])

        url = self.request.route_url("printproxy_report_create", format="pdf")
        url, headers = build_url("Check the printproxy request (create)", url, self.request, {
            "Content-Type": "application/json;charset=utf-8",
        })

        h = Http()
        resp, content = h.request(url, "POST", headers=headers, body=body)

        if resp.status != httplib.OK:
            self.set_status(resp.status, resp.reason)
            return "Failed creating the print job: " + content

        job = loads(content)
        url = self.request.route_url("printproxy_status", ref=job["ref"])
        url, headers = build_url("Check the printproxy pdf statur", url, self.request)
        done = False
        while not done:
            sleep(1)
            resp, content = h.request(url, headers=headers)
            if resp.status != httplib.OK:
                self.set_status(resp.status, resp.reason)
                return "Failed get the status: " + content

            status = loads(content)
            if "error" in status:
                self.set_status(500, status["error"])
                return "Failed to do the printing: %s" % status["error"]
            done = status["done"]

        url = self.request.route_url("printproxy_report_get", ref=job["ref"])
        url, headers = build_url("Check the printproxy pdf retrieve", url, self.request)
        resp, content = h.request(url, headers=headers)

        if resp.status != httplib.OK:
            self.set_status(resp.status, resp.reason)
            return "Failed to get the PDF: " + content

        return "OK"

    @view_config(route_name="checker_fts")
    def fts(self):
        return self.make_response(self._fts())

    def _fts(self):
        url = add_url_params(self.request.route_url("fulltextsearch"), {
            "query": self.settings["fulltextsearch"],
            "limit": "1",
        })
        url, headers = build_url("Check the fulltextsearch", url, self.request)

        h = Http()
        resp, content = h.request(url, headers=headers)

        if resp.status != httplib.OK:
            self.set_status(resp.status, resp.reason)
            return content

        result = loads(content)

        if len(result["features"]) == 0:
            self.set_status(httplib.BAD_REQUEST, httplib.responses[httplib.BAD_REQUEST])
            return "No result"

        return "OK"

    @view_config(route_name="checker_theme_errors")
    def themes_errors(self):
        from c2cgeoportal.models import DBSession, Interface

        settings = self.settings.get("themes", {})

        url = self.request.route_url("themes")
        h = Http()
        default_params = settings.get("default", {}).get("params", {})
        results = []
        for interface, in DBSession.query(Interface.name).all():
            params = {}
            params.update(default_params)
            params.update(settings.get(interface, {}).get("params", {}))
            params["interface"] = interface
            interface_url = add_url_params(url, params)

            interface_url, headers = build_url("Check the theme", interface_url, self.request)

            resp, content = h.request(interface_url, headers=headers)

            if resp.status != httplib.OK:
                self.set_status(resp.status, resp.reason)
                results.append("{}: {}".format(interface, content))

            result = loads(content)

            if len(result["errors"]) != 0:
                self.set_status(500, "Theme with error")

                results.append("{}: Theme with error\n{}".format(
                    Interface.name,
                    "\n".join(result["errors"])
                ))

        return self.make_response(
            "OK" if len(results) == 0 else urllib.unquote("\n\n".join(results))
        )

    @view_config(route_name="checker_lang_files")
    def checker_lang_files(self):
        available_locale_names = self.request.registry.settings["available_locale_names"]

        if self.request.registry.settings["default_locale_name"] not in available_locale_names:
            self.set_status(500, "default_locale_name not in available_locale_names")

            return self.make_response((
                "Your `default_locale_names` '%s' is not in your "
                "`available_locale_names` '%s'" % (
                    self.request.registry.settings["default_locale_name"],
                    ", ".join(available_locale_names)
                )
            ))

        named_urls = []
        for _type in self.settings.get("lang_files", []):
            for lang in available_locale_names:
                if _type == "cgxp":
                    named_urls.append((
                        "%s-%s" % (_type, lang),
                        self.request.static_url(
                            "{package}:static/build/lang-{lang}.js".format(
                                package=self.request.registry.settings["package"], lang=lang
                            )
                        )
                    ))
                elif _type == "cgxp-api":
                    named_urls.append((
                        "%s-%s" % (_type, lang),
                        self.request.static_url(
                            "{package}:static/build/api-lang-{lang}.js".format(
                                package=self.request.registry.settings["package"], lang=lang
                            )
                        )
                    ))
                elif _type == "ngeo":
                    named_urls.append((
                        "%s-%s" % (_type, lang),
                        self.request.static_url(
                            "{package}:static-ngeo/build/{lang}.json".format(
                                package=self.request.registry.settings["package"], lang=lang
                            )
                        )
                    ))
                else:
                    self.set_status(500, "Unknown lang_files")
                    return self.make_response((
                        "Your language type value '%s' isn't valid, "
                        "available values [cgxp, cgxp-api, ngeo]" % (
                            _type
                        )
                    ))
        return self.testurls(named_urls)

    @view_config(route_name="checker_phantomjs")
    def checker_phantomjs(self):
        package = self.request.registry.settings["package"]

        base_path = os.path.dirname(os.path.dirname(
            os.path.abspath(__import__(package).__file__)))

        phantomjs_base_path = "node_modules/.bin/phantomjs"
        executable_path = os.path.join(base_path, phantomjs_base_path)

        checker_config_base_path = "node_modules/ngeo/buildtools/check-example.js"
        checker_config_path = os.path.join(base_path, checker_config_base_path)

        results = []
        for route in self.settings["phantomjs_routes"]:
            url = self.request.route_url(route)

            args = [executable_path, "--local-to-remote-url-access=true", checker_config_path, url]

            # check_output throws a CalledProcessError if return code is > 0
            try:
                check_output(args)
                "{}: OK".format(route)
            except CalledProcessError as e:
                "{}: {}".format(route, results.append(e.output))

        return self.make_response(
            "OK" if len(results) == 0 else urllib.unquote("\n\n".join(results))
        )



# Copyright (c) 2014-2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

"""Add restricted theme

Revision ID: 1d5d4abfebd1
Revises: 54645a535ad6
Create Date: 2014-11-25 16:51:51.567026
"""

from alembic import op, context
from sqlalchemy import Column, ForeignKey
from sqlalchemy.types import Integer, Boolean

# revision identifiers, used by Alembic.
revision = '1d5d4abfebd1'
down_revision = '54645a535ad6'


def upgrade():
    schema = context.get_context().config.get_main_option('schema')

    engine = op.get_bind().engine
    if type(engine).__name__ != 'MockConnection' and \
            op.get_context().dialect.has_table(
                engine, 'restricted_role_theme', schema=schema):  # pragma: nocover
        return

    op.add_column('theme', Column(
        'public', Boolean, server_default='t', nullable=False
    ), schema=schema)
    op.create_table(
        'restricted_role_theme',
        Column(
            'role_id', Integer, ForeignKey(schema + '.role.id'), primary_key=True
        ),
        Column(
            'theme_id', Integer, ForeignKey(schema + '.theme.id'), primary_key=True
        ),
        schema=schema
    )


def downgrade():
    schema = context.get_context().config.get_main_option('schema')

    op.drop_table('restricted_role_theme', schema=schema)
    op.drop_column('theme', 'public', schema=schema)



# -*- coding: utf-8 -*-

# Copyright (c) 2012-2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

from base64 import b64encode
import json

from pyramid.httpexceptions import HTTPBadRequest
from pyramid.response import Response
from pyramid.view import view_config

from c2cgeoportal.lib.caching import set_common_headers, NO_CACHE


def json_base64_encode(filename, file):
    """
    Generate a JSON-wrapped base64-encoded string.
    See http://en.wikipedia.org/wiki/Base64
    """
    yield '{"filename":%s,"data":"' % (json.dumps(filename),)
    yield b64encode(file.read())
    yield '","success":true}'


@view_config(route_name="echo")
def echo(request):
    """
    Echo an uploaded file back to the client as an text/html document so it can
    be handled by Ext.

    The response is JSON-wrapped and base64-encoded to ensure that there are no
    special HTML characters or charset problems and so that braindead ext
    doesn't barf on it.

    See:
    http://docs.sencha.com/ext-js/3-4/#!/api/Ext.form.BasicForm-cfg-fileUpload
    """
    try:
        file = request.POST["file"]
    except KeyError:
        return HTTPBadRequest()
    response = Response()
    response.app_iter = json_base64_encode(file.filename, file.file)
    return set_common_headers(
        request, "echo", NO_CACHE,
        response=response, content_type="text/html"
    )



# -*- coding: utf-8 -*-

# Copyright (c) 2014-2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


import os
import re
import sys
import shutil
import argparse
import httplib2
import yaml
from subprocess import check_call, CalledProcessError
from argparse import ArgumentParser
from alembic.config import Config
from alembic import command
from c2cgeoportal.lib.bashcolor import colorize, GREEN, YELLOW, RED

try:
    from subprocess import check_output
except ImportError:
    from subprocess import Popen, PIPE

    def check_output(cmd, cwd=None, stdin=None, stderr=None, shell=False):  # noqa
        """Backwards compatible check_output"""
        p = Popen(cmd, cwd=cwd, stdin=stdin, stderr=stderr, shell=shell, stdout=PIPE)
        out, err = p.communicate()
        return out

VERSION_RE = "^[0-9]+\.[0-9]+\..+$"


def main():
    """
    tool used to help th use in the user tash
    """

    usage = """usage: {prog} [command] [options]

Available commands:

""" + colorize("help", GREEN) + """: show this page
""" + colorize("upgrade", GREEN) + """: upgrade the application to a new version
""" + colorize("deploy", GREEN) + """: deploy the application to a server

To have some help on a command type:
{prog} help [command]""".format(prog=sys.argv[0])

    if len(sys.argv) <= 1:
        print(usage)
        exit()

    if sys.argv[1] == "help":
        if len(sys.argv) > 2:
            parser = _fill_arguments(sys.argv[2])
            parser.print_help()
        else:
            print(usage)
        exit()

    parser = _fill_arguments(sys.argv[1])
    options = parser.parse_args(sys.argv[2:])

    c2ctool = C2cTool(options)
    if sys.argv[1] == "upgrade":
        c2ctool.upgrade()
    elif sys.argv[1] == "deploy":
        c2ctool.deploy()
    else:
        print("Unknown command")


def _fill_arguments(command):
    parser = ArgumentParser(prog="%s %s" % (sys.argv[0], command), add_help=False)
    parser.add_argument(
        "--no-cleanall",
        help="Run clean instead of cleanall",
        default="cleanall",
        action="store_const",
        const="clean",
        dest="clean",
    )
    parser.add_argument(
        "--windows",
        action="store_true",
        help="Use the windows c2cgeoportal package",
    )
    parser.add_argument(
        "--git-remote",
        metavar="GITREMOTE",
        help="Specify the remote branch",
        default="origin",
    )
    parser.add_argument(
        "--index-url",
        help="no more used",
    )
    parser.add_argument(
        "--c2cgeoportal-url",
        help="no more used",
    )

    if command == "help":
        parser.add_argument(
            "command", metavar="COMMAND", help="The command"
        )
    elif command == "upgrade":
        parser.add_argument(
            "file", metavar="MAKEFILE", help="The makefile used to build", default=None
        )
        parser.add_argument(
            "--step", type=int, help=argparse.SUPPRESS, default=0
        )
        parser.add_argument(
            "version", metavar="VERSION", help="Upgrade to version"
        )
    elif command == "deploy":
        parser.add_argument(
            "host", metavar="HOST", help="The destination host"
        )
        parser.add_argument(
            "--components",
            help="Restrict component to update. [databases,files,code]. default to all",
            default=None
        )
    else:
        print("Unknown command")
        exit()

    return parser


class C2cTool:

    color_bar = colorize("================================================================", GREEN)

    def print_step(self, step, intro="To continue type:"):
        print(intro)
        print(colorize("make -f %s upgrade%i", YELLOW) % (
            self.options.file if self.options.file is not None else "<user.mk>",
            step
        ))

    def get_project(self):
        if not os.path.isfile("project.yaml"):
            print("Unable to find the required 'project.yaml' file.")
            exit(1)

        with open("project.yaml", "r") as f:
            return yaml.load(f)

    def test_checkers(self):
        http = httplib2.Http()
        for check_type in ("", "type=all"):
            resp, content = http.request(
                "http://localhost%s%s" % (self.project["checker_path"], check_type),
                method="GET",
                headers={
                    "Host": self.project["host"]
                }
            )
            if resp.status < 200 or resp.status >= 300:
                print(self.color_bar)
                print("Checker error:")
                print("Open `http://%s%s%s` for more informations." % (
                    self.project["host"], self.project["checker_path"], check_type
                ))
                return False
        return True

    def __init__(self, options):
        self.options = options
        self.project = self.get_project()

    def upgrade(self):
        self.venv_bin = ".build/venv/bin"
        if self.options.windows:
            self.options.clean = "clean"
            self.venv_bin = ".build/venv/Scripts"

        if self.options.step == 0:
            self.step0()
        elif self.options.step == 1:
            self.step1()
        elif self.options.step == 2:
            self.step2()
        elif self.options.step == 3:
            self.step3()
        elif self.options.step == 4:
            self.step4()

    def step0(self):
        if re.match(VERSION_RE, self.options.version) is not None:
            http = httplib2.Http()
            url = (
                "http://raw.github.com/camptocamp/c2cgeoportal/%s/"
                "c2cgeoportal/scaffolds/update/CONST_versions_requirements.txt" %
                self.options.version
            )
            headers, _ = http.request(url)
            if headers.status != 200:
                print("Failed downloading the c2cgeoportal CONST_versions_requirements.txt file.")
                print(url)
                exit(1)

        if os.path.split(os.path.realpath("."))[1] != self.project["project_folder"]:
            print("Your project isn't in the right folder!")
            print("It should be in folder '%s' instead of folder '%s'." % (
                self.project["project_folder"], os.path.split(os.path.realpath("."))[1]
            ))
            exit(1)

        check_call(["git", "status"])
        print("")
        print(self.color_bar)
        print(
            "Here is the output of 'git status'. Please make sure to commit all your changes "
            "before going further. All uncommited changes will be lost."
        )
        self.print_step(1)

    def step1(self):
        notes = []

        check_call(["git", "reset", "--hard"])
        check_call(["git", "clean", "--force", "-d"])
        check_call(["git", "submodule", "foreach", "--recursive", "git", "reset", "--hard"])
        check_call(["git", "submodule", "foreach", "--recursive", "git", "clean", "--force", "-d"])

        branch = check_output(["git", "rev-parse", "--abbrev-ref", "HEAD"]).strip()
        # remove all no more existing branches
        check_call(["git", "fetch", "origin", "--prune"])
        branches = check_output(["git", "branch", "--all"]).split("\n")
        if "  remotes/origin/%s" % branch in branches:
            try:
                check_call(["git", "pull", "--rebase", self.options.git_remote, branch])
            except CalledProcessError:
                print(self.color_bar)
                print(colorize("The pull (rebase) failed.", RED))
                print(colorize("Please solve the rebase and run the step again.", YELLOW))
                exit(1)

        check_call(["git", "submodule", "sync"])
        check_call(["git", "submodule", "update", "--init"])
        check_call(["git", "submodule", "foreach", "git", "submodule", "sync"])
        check_call(["git", "submodule", "foreach", "git", "submodule", "update", "--init"])

        if len(check_output(["git", "status", "-z"]).strip()) != 0:
            print(self.color_bar)
            print(colorize("The pull isn't fast forward.", RED))
            print(colorize("Please solve the rebase and run the step again.", YELLOW))
            exit(1)

        check_call(["git", "submodule", "foreach", "git", "submodule", "sync"])
        check_call(["git", "submodule", "foreach", "git", "submodule", "update", "--init"])

        if not self.options.windows:
            check_call(["make", "-f", self.options.file, ".build/requirements.timestamp"])
            pip_cmd = ["%s/pip" % self.venv_bin, "install"]
            if self.options.version == "master":
                check_call(["%s/pip" % self.venv_bin, "uninstall", "--yes", "c2cgeoportal"])
                pip_cmd += ["--pre", "c2cgeoportal"]
            else:
                pip_cmd += ["c2cgeoportal==%s" % (self.options.version)]
            check_call(pip_cmd)

        check_call([
            "%s/pcreate" % self.venv_bin, "--ignore-conflicting-name", "--overwrite",
            "--scaffold=c2cgeoportal_update", "../%s" % self.project["project_folder"]
        ])
        pcreate_cmd = [
            "%s/pcreate" % self.venv_bin, "--ignore-conflicting-name",
            "--scaffold=c2cgeoportal_create", "/tmp/%s" % self.project["project_folder"],
        ]
        for name, value in self.project["template_vars"].items():
            if isinstance(value, basestring):
                value = value.encode('utf-8')
            pcreate_cmd.append("{}={}".format(name, value))
        check_call(pcreate_cmd)
        check_call(["make", "-f", self.options.file, self.options.clean])

        diff_file = open("changelog.diff", "w")
        check_call(["git", "diff", "CONST_CHANGELOG.txt"], stdout=diff_file)
        diff_file.close()

        check_call(["make", "-f", self.options.file, "update-node-modules"])
        check_call(["make", "-f", self.options.file, ".build/requirements.timestamp"])

        print("")
        print(self.color_bar)
        print("\n".join(notes))
        print(
            "Apply the manual migration steps based on what is in the CONST_CHANGELOG.txt file"
            " (listed in the `changelog.diff` file)."
        )
        self.print_step(2)

    def step2(self):
        if self.options.file is None:
            print("The makefile is missing")
            exit(1)

        if os.path.isfile("changelog.diff"):
            os.unlink("changelog.diff")

        check_call(["make", "-f", self.options.file, "build"])

        command.upgrade(Config("alembic.ini"), "head")
        command.upgrade(Config("alembic_static.ini"), "head")

        if not self.options.windows:
            check_call(self.project.get("cmds", {}).get(
                "apache_graceful",
                ["sudo", "/usr/sbin/apache2ctl", "graceful"]
            ))

        print("")
        print(self.color_bar)
        print("The upgrade is nearly done, now you should:")
        print("- Test your application.")

        if self.options.windows:
            print("You are running on Windows, please restart your Apache server,")
            print("because we can not do that automatically.")

        self.print_step(3)

    def step3(self):
        if not self.test_checkers():
            self.print_step(3, intro="Correct them, then type:")
            exit(1)

        if os.path.exists("/tmp/%s" % self.project["project_folder"]):
            shutil.rmtree("/tmp/%s" % self.project["project_folder"])

        # Required to remove from the Git stage the ignored file when we lunch the step again
        check_call(["git", "reset", "--mixed"])

        check_call(["git", "add", "-A"])
        check_call(["git", "status"])

        print("")
        print(self.color_bar)
        print("We will commit all the above files!")
        print(
            "If there are some files which should not be commited, then you should "
            "add them into the `.gitignore` file and launch step 3 again."
        )

        self.print_step(4, intro="Then to commit your changes type:")

    def step4(self):
        check_call(["git", "commit", "-m", "Upgrade to GeoMapFish %s" % self.options.version])

        print("")
        print(self.color_bar)
        print("")
        print(colorize("Congratulations your upgrade is a success.", GREEN))
        print("")
        branch = check_output(["git", "rev-parse", "--abbrev-ref", "HEAD"]).strip()
        print("Now all your files will be commited, you should do a git push %s %s." % (
            self.options.git_remote, branch
        ))

    def deploy(self):
        if not self.test_checkers():
            print(colorize("Correct them and run again", RED))
            exit(1)

        check_call(["sudo", "-u", "deploy", "deploy", "-r", "deploy/deploy.cfg", self.options.host])
        check_call(["make", "-f", self.options.file, "build"])


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# Copyright (c) 2011-2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


import httplib2
import subprocess

import yaml
from json import loads
from urlparse import urlsplit
from xml.dom.minidom import parseString
from sqlalchemy.exc import ProgrammingError

from lingua.extractors import Extractor
from lingua.extractors import Message
from pyramid.paster import bootstrap

from c2cgeoportal.lib import add_url_params
from c2cgeoportal.lib.print_ import *  # noqa
from c2cgeoportal.lib.bashcolor import colorize, RED


class GeoMapfishAngularExtractor(Extractor):  # pragma: nocover
    "GeoMapfish angular extractor"

    extensions = [".js", ".html"]

    def __call__(self, filename, options):
        message_str = subprocess.check_output(["node", "tools/extract-messages.js", filename])
        try:
            messages = []
            for contexts, message in loads(message_str):
                for context in contexts.split(", "):
                    messages.append(Message(
                        None, message, None, [], u"", u"", context.split(":")
                    ))
            return messages
        except:
            print(colorize("An error occurred", RED))
            print(colorize(message_str, RED))
            print("------")
            raise


class GeoMapfishConfigExtractor(Extractor):  # pragma: nocover
    "GeoMapfish config extractor (raster layers, and print templates)"

    extensions = [".yaml"]

    def __call__(self, filename, options):
        with open(filename) as config_file:
            config = yaml.load(config_file)
            # for application config (.build/config.yaml)
            if "vars" in config:
                return [
                    Message(
                        None, raster_layer, None, [], u"", u"",
                        (filename, u"raster/%s" % raster_layer)
                    )
                    for raster_layer in config["vars"].get("raster", {}).keys()
                ]
            # for the print config
            elif "templates" in config:
                result = []
                for template in config.get("templates").keys():
                    result.append(Message(
                        None, template, None, [], u"", u"",
                        (filename, u"template/%s" % template)
                    ))
                    result += [
                        Message(
                            None, attribute, None, [], u"", u"",
                            (filename, u"template/%s/%s" % (template, attribute))
                        )
                        for attribute in config.get("templates")[template].attributes.keys()
                    ]
                return result
            else:
                raise Exception("Not a known config file")


class GeoMapfishThemeExtractor(Extractor):  # pragma: nocover
    "GeoMapfish theme extractor"

    # Run on the development.ini file
    extensions = [".ini"]
    featuretype_cache = {}

    def __call__(self, filename, options):
        messages = []

        self.env = bootstrap(filename)
        with open("project.yaml") as f:
            self.package = yaml.load(f)

        try:
            from c2cgeoportal.models import Theme, LayerGroup, \
                LayerWMS, LayerWMTS

            self._import(Theme, messages)
            self._import(LayerGroup, messages)
            self._import(LayerWMS, messages, self._import_layer_wms)
            self._import(LayerWMTS, messages, self._import_layer_wmts)
        except ProgrammingError as e:
            print(colorize("ERROR: The database is probably not up to date", RED))
            print(colorize(e, RED))

        return messages

    def _import(self, object_type, messages, callback=None):
        from c2cgeoportal.models import DBSession

        items = DBSession.query(object_type)
        for item in items:
            messages.append(Message(
                None, item.name, None, [], u"", u"",
                (item.item_type, item.name)
            ))

            if callback is not None:
                callback(item, messages)

    def _import_layer_wms(self, layer, messages):
        server = layer.server_ogc
        url = server.url_wfs or server.url or \
            self.env["registry"].settings["mapserverproxy"]["mapserv_url"]
        for wms_layer in layer.layer.split(","):
            self._import_layer_attributes(
                url, wms_layer, layer.item_type, layer.name, layer.id, messages
            )

    def _import_layer_wmts(self, layer, messages):
        layers = [d.value for d in layer.ui_metadatas if d.name == "wmsLayer"]
        url = [d.value for d in layer.ui_metadatas if d.name == "wmsUrl"]
        if len(url) == 1 and len(layers) >= 1:
            for wms_layer in layers:
                self._import_layer_attributes(
                    url[0], wms_layer, layer.item_type, layer.name, layer.id, messages
                )

    def _import_layer_attributes(self, url, layer, item_type, name, layer_id, messages):
        for attribute in self._layer_attributes(url, layer):
            messages.append(Message(
                None, attribute, None, [], "", "", (".".join([item_type, name]), layer)
            ))

    def _layer_attributes(self, url, layer):
        url = add_url_params(url, {
            "SERVICE": "WFS",
            "VERSION": "1.1.0",
            "REQUEST": "DescribeFeatureType",
        })

        if url not in self.featuretype_cache:
            print("Get DescribeFeatureType for url: %s" % url)
            self.featuretype_cache[url] = None

            # forward request to target (without Host Header)
            http = httplib2.Http()
            h = {}
            if urlsplit(url).hostname == "localhost":  # pragma: no cover
                h["Host"] = self.package["host"]
            try:
                resp, content = http.request(url, method="GET", headers=h)
            except:  # pragma: no cover
                print("Unable to DescribeFeatureType from url %s" % url)
                self.capabilities_cache[url] = None
                return []

            if resp.status < 200 or resp.status >= 300:  # pragma: no cover
                print(
                    "DescribeFeatureType from url %s return the error: %i %s" %
                    (url, resp.status, resp.reason)
                )
                self.capabilities_cache[url] = None
                return []

            try:
                describe = parseString(content)
                self.featuretype_cache[url] = describe
            except AttributeError:
                print(
                    "WARNING! an error occured while trying to "
                    "read the mapfile and recover the themes."
                )
                print("url: %s\nxml:\n%s" % (url, content))
        else:
            describe = self.featuretype_cache[url]

        if describe is None:
            return []

        attributes = []
        # Should probably be adapted for other king of servers
        for type_element in describe.getElementsByTagName("complexType"):
            if type_element.getAttribute("name") == "%sType" % layer:
                for element in type_element.getElementsByTagName("element"):
                    if not element.getAttribute("type").startswith("gml:"):
                        attributes.append(element.getAttribute("name"))
        return attributes



# -*- coding: utf-8 -*-

# Copyright (c) 2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

import importlib
from pyramid.view import view_config


def _add_version_info(package, orig):
    """
    Try to add the version information from the given package
    """
    try:
        info = importlib.import_module(package + ".version").INFO
        orig[package] = info
    except:
        pass


def _get_versions(request):
    result = {}
    _add_version_info("c2cgeoportal", result)
    package = request.registry.settings.get("package", None)
    if package is not None:
        _add_version_info(package, result)
    return result


@view_config(route_name="version_json", renderer="json")
def version_json(request):
    return _get_versions(request)



# -*- coding: utf-8 -*-

# Copyright (c) 2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

import os
import re
import subprocess


def _get_versions():
    version = subprocess.check_output(["python", "./setup.py", "--version"]).strip()
    git_hash = subprocess.check_output(["git", "rev-parse", "HEAD"]).strip()
    try:
        with open(os.devnull, "w") as devnull:
            git_tag = subprocess.check_output(["git", "describe", "--tags", "--first-parent"],
                                              stderr=devnull).strip()
        git_tag = re.sub(r"-g[a-f0-9]+$", "", git_tag)
    except:
        git_tag = None
    return {
        "version": version,
        "git_hash": git_hash,
        "git_tag": git_tag
    }


def main():
    print "INFO = {"
    for key, value in _get_versions().iteritems():
        print "    \"%s\": \"%s\"," % (key, str(value).replace("\\", "\\\\").replace("\"", "\\\""))
    print "}"


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# Copyright (c) 2011-2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.


import logging
try:
    from hashlib import sha1
    sha1  # suppress pyflakes warning
except ImportError:  # pragma: nocover
    from sha import new as sha1

import sqlahelper
from papyrus.geo_interface import GeoInterface
from sqlalchemy import ForeignKey, Table, event
from sqlalchemy.types import Integer, Boolean, Unicode, Float, String, \
    Enum, DateTime, UserDefinedType
from sqlalchemy.schema import Index
from sqlalchemy.orm import relationship, backref
from sqlalchemy.exc import UnboundExecutionError
from geoalchemy2 import Geometry, func
from geoalchemy2.shape import to_shape
from formalchemy import Column
from pyramid.security import Allow, ALL_PERMISSIONS, DENY_ALL
from pyramid.i18n import TranslationStringFactory

from c2cgeoportal import schema, parentschema, srid
from c2cgeoportal.lib import caching
from c2cgeoportal.lib.sqlalchemy_ import JSONEncodedDict

__all__ = [
    "Base", "DBSession", "Functionality", "User", "Role", "TreeItem",
    "TreeGroup", "LayerGroup", "Theme", "Layer", "RestrictionArea",
    "LayerV1", "ServerOGC",
    "LayerWMS", "LayerWMTS", "Interface", "UIMetadata", "WMTSDimension",
    "LayergroupTreeitem"
]

_ = TranslationStringFactory("c2cgeoportal")
log = logging.getLogger(__name__)

Base = sqlahelper.get_base()
DBSession = sqlahelper.get_session()

DBSessions = {
    "dbsession": DBSession,
}

try:
    postgis_version = DBSession.execute(func.postgis_version()).scalar()
except UnboundExecutionError:  # pragma: nocover - needed by non functional tests
    postgis_version = "2.0"
management = postgis_version.startswith("1.")

AUTHORIZED_ROLE = "role_admin"

if schema is not None:
    _schema = schema
else:  # pragma: nocover
    raise Exception(
        "schema not specified, you need to add it to your config"
    )
_parentschema = parentschema

if srid is not None:
    _srid = srid
else:  # pragma: nocover
    raise Exception(
        "srid not specified, you need to add it to your config"
    )


def cache_invalidate_cb(*args):
    caching.invalidate_region()


class TsVector(UserDefinedType):
    """ A custom type for PostgreSQL's tsvector type. """
    def get_col_spec(self):  # pragma: nocover
        return "TSVECTOR"


class FullTextSearch(GeoInterface, Base):
    __tablename__ = "tsearch"
    __table_args__ = (
        Index("tsearch_ts_idx", "ts", postgresql_using="gin"),
        {"schema": _schema}
    )
    __acl__ = [DENY_ALL]

    id = Column(Integer, primary_key=True)
    label = Column(Unicode)
    layer_name = Column(Unicode)  # Deprecated in v2
    role_id = Column(Integer, ForeignKey(_schema + ".role.id"), nullable=True)
    role = relationship("Role")
    interface_id = Column(Integer, ForeignKey(_schema + ".interface.id"), nullable=True)
    interface = relationship("Interface")
    lang = Column(String(2), nullable=True)
    public = Column(Boolean, server_default="true")
    ts = Column(TsVector)
    the_geom = Column(Geometry("GEOMETRY", srid=_srid, management=management))
    params = Column(JSONEncodedDict, nullable=True)
    actions = Column(JSONEncodedDict, nullable=True)
    from_theme = Column(Boolean, server_default="false")


class Functionality(Base):
    __label__ = _(u"Functionality")
    __plural__ = _(u"Functionalities")
    __tablename__ = "functionality"
    __table_args__ = {"schema": _schema}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]

    id = Column(Integer, primary_key=True)
    name = Column(Unicode, nullable=False, label=_(u"Name"))
    value = Column(Unicode, nullable=False, label=_(u"Value"))
    description = Column(Unicode)

    def __init__(self, name=u"", value=u"", description=u""):
        self.name = name
        self.value = value
        self.description = description

    def __unicode__(self):
        return "%s - %s" % (self.name or u"", self.value or u"")  # pragma: nocover

event.listen(Functionality, "after_update", cache_invalidate_cb)
event.listen(Functionality, "after_delete", cache_invalidate_cb)

# association table role <> functionality
role_functionality = Table(
    "role_functionality", Base.metadata,
    Column(
        "role_id", Integer,
        ForeignKey(_schema + ".role.id"), primary_key=True
    ),
    Column(
        "functionality_id", Integer,
        ForeignKey(_schema + ".functionality.id"), primary_key=True
    ),
    schema=_schema
)

# association table theme <> functionality
theme_functionality = Table(
    "theme_functionality", Base.metadata,
    Column(
        "theme_id", Integer,
        ForeignKey(_schema + ".theme.id"), primary_key=True
    ),
    Column(
        "functionality_id", Integer,
        ForeignKey(_schema + ".functionality.id"), primary_key=True
    ),
    schema=_schema
)


class User(Base):
    __label__ = _(u"User")
    __plural__ = _(u"Users")
    __tablename__ = "user"
    __table_args__ = {"schema": _schema + "_static"}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]
    item_type = Column("type", String(10), nullable=False)
    __mapper_args__ = {
        "polymorphic_on": item_type,
        "polymorphic_identity": "user",
    }

    id = Column(Integer, primary_key=True)
    username = Column(
        Unicode, unique=True, nullable=False,
        label=_(u"Username")
    )
    _password = Column(
        "password", Unicode, nullable=False,
        label=_(u"Password")
    )
    temp_password = Column(
        "temp_password", Unicode, nullable=True,
        label=_(u"Password")
    )
    email = Column(Unicode, nullable=False, label=_(u"E-mail"))
    is_password_changed = Column(Boolean, default=False, label=_(u"Password Changed"))

    role_name = Column(String, label=_(u"Role"))

    _cached_role_name = None
    _cached_role = None

    @property
    def role(self):
        if self._cached_role_name == self.role_name:
            return self._cached_role

        if self.role_name is None or self.role_name == "":  # pragma: no cover
            self._cached_role_name = self.role_name
            self._cached_role = None
            return None

        result = self._sa_instance_state.session.query(Role).filter(
            Role.name == self.role_name
        ).all()
        if len(result) == 0:  # pragma: no cover
            self._cached_role = None
        else:
            self._cached_role = result[0]

        self._cached_role_name = self.role_name
        return self._cached_role

    if _parentschema is not None and _parentschema != "":  # pragma: no cover
        # parent role relationship
        parent_role_name = Column(String, label=_(u"Parent Role"))

        @property
        def parent_role(self):
            return self._sa_instance_state.session.query(Role).filter(
                Role.name == self.parent_role_name
            ).one()

    def __init__(
        self, username=u"", password=u"", email=u"", is_password_changed=False,
        functionalities=[], role=None
    ):
        self.username = username
        self.password = password
        self.email = email
        self.is_password_changed = is_password_changed
        self.functionalities = functionalities
        if role is not None:
            self.role_name = role.name

    def _get_password(self):
        """returns password"""
        return self._password

    def _set_password(self, password):
        """encrypts password on the fly."""
        self._password = self.__encrypt_password(password)

    def set_temp_password(self, password):
        """encrypts password on the fly."""
        self.temp_password = self.__encrypt_password(password)

    def __encrypt_password(self, password):
        """Hash the given password with SHA1."""
        return sha1(password.encode("utf8")).hexdigest()

    def validate_password(self, passwd):
        """Check the password against existing credentials.
        this method _MUST_ return a boolean.

        @param passwd: the password that was provided by the user to
        try and authenticate. This is the clear text version that we will
        need to match against the (possibly) encrypted one in the database.
        @type password: string
        """
        if self._password == self.__encrypt_password(passwd):
            return True
        if \
                self.temp_password is not None and \
                self.temp_password != "" and \
                self.temp_password == self.__encrypt_password(passwd):
            self._password = self.temp_password
            self.temp_password = None
            self.is_password_changed = True
            return True
        return False

    password = property(_get_password, _set_password)

    def __unicode__(self):
        return self.username or u""  # pragma: nocover


class Role(Base):
    __label__ = _(u"Role")
    __plural__ = _(u"Roles")
    __tablename__ = "role"
    __table_args__ = {"schema": _schema}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]

    id = Column(Integer, primary_key=True)
    name = Column(Unicode, unique=True, nullable=False, label=_(u"Name"))
    description = Column(Unicode, label=_(u"Description"))
    extent = Column(Geometry("POLYGON", srid=_srid, management=management))

    # functionality
    functionalities = relationship(
        "Functionality", secondary=role_functionality,
        cascade="save-update,merge,refresh-expire"
    )

    def __init__(self, name=u"", description=u"", functionalities=[], extent=None):
        self.name = name
        self.functionalities = functionalities
        self.extent = extent
        self.description = description

    def __unicode__(self):
        return self.name or u""  # pragma: nocover

    @property
    def bounds(self):
        if self.extent is None:
            return None
        return to_shape(self.extent).bounds

event.listen(Role.functionalities, "set", cache_invalidate_cb)
event.listen(Role.functionalities, "append", cache_invalidate_cb)
event.listen(Role.functionalities, "remove", cache_invalidate_cb)


class TreeItem(Base):
    __tablename__ = "treeitem"
    __table_args__ = {"schema": _schema}
    __acl__ = [DENY_ALL]
    item_type = Column("type", String(10), nullable=False)
    __mapper_args__ = {"polymorphic_on": item_type}

    id = Column(Integer, primary_key=True)
    name = Column(Unicode, label=_(u"Name"))
    metadata_url = Column(Unicode, label=_(u"Metadata URL"))  # shouldn't be used in V2
    description = Column(Unicode, label=_(u"Description"))

    @property
    def parents(self):  # pragma: nocover
        return [c.group for c in self.parents_relation]

    def is_in_interface(self, name):
        if not hasattr(self, "interfaces"):  # pragma: nocover
            return False

        for interface in self.interfaces:
            if interface.name == name:
                return True

        return False

    def get_metadatas(self, name):  # pragma: nocover
        return [metadata for metadata in self.ui_metadatas if metadata.name == name]

    def __init__(self, name=u""):
        self.name = name

    def __unicode__(self):
        return self.name or u""  # pragma: nocover

event.listen(TreeItem, "after_insert", cache_invalidate_cb, propagate=True)
event.listen(TreeItem, "after_update", cache_invalidate_cb, propagate=True)
event.listen(TreeItem, "after_delete", cache_invalidate_cb, propagate=True)


# association table LayerGroup <> TreeItem
class LayergroupTreeitem(Base):
    __label__ = _(u"Tree links order")
    __plural__ = _(u"Tree links order")
    __tablename__ = "layergroup_treeitem"
    __table_args__ = {"schema": _schema}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]

    # required by formalchemy
    id = Column(Integer, primary_key=True)
    description = Column(Unicode, label=_(u"Description"))
    treegroup_id = Column(
        Integer, ForeignKey(_schema + ".treegroup.id")
    )
    group = relationship(
        "TreeGroup",
        backref=backref(
            "children_relation",
            order_by="LayergroupTreeitem.ordering",
            cascade="save-update,merge,delete,delete-orphan",
        ),
        primaryjoin="LayergroupTreeitem.treegroup_id==TreeGroup.id",
    )
    treeitem_id = Column(
        Integer, ForeignKey(_schema + ".treeitem.id")
    )
    item = relationship(
        "TreeItem",
        backref=backref(
            "parents_relation", cascade="save-update,merge,delete,delete-orphan"
        ),
        primaryjoin="LayergroupTreeitem.treeitem_id==TreeItem.id",
    )
    ordering = Column(Integer)

    # Used by formalchemy
    def __unicode__(self):  # pragma: nocover
        return self.group.name

    def __init__(self, group=None, item=None, ordering=0):
        self.group = group
        self.item = item
        self.ordering = ordering

event.listen(LayergroupTreeitem, "after_insert", cache_invalidate_cb, propagate=True)
event.listen(LayergroupTreeitem, "after_update", cache_invalidate_cb, propagate=True)
event.listen(LayergroupTreeitem, "after_delete", cache_invalidate_cb, propagate=True)


class TreeGroup(TreeItem):
    __tablename__ = "treegroup"
    __table_args__ = {"schema": _schema}
    __acl__ = [DENY_ALL]

    id = Column(
        Integer, ForeignKey(_schema + ".treeitem.id"), primary_key=True
    )

    def _get_children(self):
        return [c.item for c in self.children_relation]

    def _set_children(self, children, order=False):
        for child in self.children_relation:
            if child.item not in children:
                child.item = None
                child.group = None
        if order is True:  # pragma: nocover
            for index, child in enumerate(children):
                current = [c for c in self.children_relation if c.item == child]
                if len(current) == 1:
                    current[0].ordering = index * 10
                else:
                    LayergroupTreeitem(self, child, index * 10)
            self.children_relation.sort(key=lambda child: child.ordering)
        else:
            current_item = [child.item for child in self.children_relation]
            for index, item in enumerate(children):
                if item not in current_item:
                    LayergroupTreeitem(self, item, 1000000 + index)
            for index, child in enumerate(self.children_relation):
                child.ordering = index * 10

    children = property(_get_children, _set_children)

    def __init__(self, name=u""):
        TreeItem.__init__(self, name=name)


class LayerGroup(TreeGroup):
    __label__ = _(u"Layer Group")
    __plural__ = _(u"Layer Groups")
    __tablename__ = "layergroup"
    __table_args__ = {"schema": _schema}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]
    __mapper_args__ = {"polymorphic_identity": "group"}

    id = Column(
        Integer, ForeignKey(_schema + ".treegroup.id"), primary_key=True
    )
    is_expanded = Column(Boolean, label=_(u"Expanded"))  # shouldn"t be used in V3
    is_internal_wms = Column(Boolean, label=_(u"Internal WMS"))
    # children have radio button instance of check box
    is_base_layer = Column(Boolean, label=_(u"Group of base layers"))  # Shouldn't be used in V3

    def __init__(
            self, name=u"", is_expanded=False,
            is_internal_wms=True, is_base_layer=False):
        TreeGroup.__init__(self, name=name)
        self.is_expanded = is_expanded
        self.is_internal_wms = is_internal_wms
        self.is_base_layer = is_base_layer


# role theme link for restricted theme
restricted_role_theme = Table(
    "restricted_role_theme", Base.metadata,
    Column(
        "role_id", Integer, ForeignKey(_schema + ".role.id"), primary_key=True
    ),
    Column(
        "theme_id", Integer,
        ForeignKey(_schema + ".theme.id"), primary_key=True
    ),
    schema=_schema
)


class Theme(TreeGroup):
    __label__ = _(u"Theme")
    __plural__ = _(u"Themes")
    __tablename__ = "theme"
    __table_args__ = {"schema": _schema}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]
    __mapper_args__ = {"polymorphic_identity": "theme"}

    id = Column(
        Integer, ForeignKey(_schema + ".treegroup.id"), primary_key=True
    )
    ordering = Column(Integer, nullable=False, label=_(u"Order"))
    public = Column(Boolean, default=True, nullable=False, label=_(u"Public"))
    icon = Column(Unicode, label=_(u"Icon"))

    # functionality
    functionalities = relationship(
        "Functionality", secondary=theme_functionality,
        cascade="save-update,merge,refresh-expire"
    )

    # restricted to role
    restricted_roles = relationship(
        "Role", secondary=restricted_role_theme,
        cascade="save-update,merge,refresh-expire",
    )

    def __init__(self, name=u"", ordering=100, icon=u""):
        TreeGroup.__init__(self, name=name)
        self.ordering = ordering
        self.icon = icon

event.listen(Theme.functionalities, "set", cache_invalidate_cb)
event.listen(Theme.functionalities, "append", cache_invalidate_cb)
event.listen(Theme.functionalities, "remove", cache_invalidate_cb)


class Layer(TreeItem):
    __tablename__ = "layer"
    __table_args__ = {"schema": _schema}
    __acl__ = [DENY_ALL]

    id = Column(
        Integer, ForeignKey(_schema + ".treeitem.id"), primary_key=True
    )
    public = Column(Boolean, default=True, label=_(u"Public"))
    geo_table = Column(Unicode, label=_(u"Related Postgres table"))
    exclude_properties = Column(Unicode, label=_(u"Attributes to exclude"))

    def __init__(self, name=u"", public=True):
        TreeItem.__init__(self, name=name)
        self.public = public


class LayerV1(Layer):  # Deprecated in v2
    __label__ = _(u"Layer")
    __plural__ = _(u"Layers")
    __tablename__ = "layerv1"
    __table_args__ = {"schema": _schema}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]
    __mapper_args__ = {"polymorphic_identity": "layerv1"}

    id = Column(
        Integer, ForeignKey(_schema + ".layer.id"), primary_key=True
    )
    is_checked = Column(Boolean, default=True, label=_(u"Checked"))  # by default
    icon = Column(Unicode, label=_(u"Icon"))  # on the tree
    layer_type = Column(Enum(
        "internal WMS",
        "external WMS",
        "WMTS",
        "no 2D",
        native_enum=False), label=_(u"Type"))
    url = Column(Unicode, label=_(u"Base URL"))  # for externals
    image_type = Column(Enum(
        "image/jpeg",
        "image/png",
        native_enum=False), label=_(u"Image type"))  # for WMS
    style = Column(Unicode, label=_(u"Style"))
    dimensions = Column(Unicode, label=_(u"Dimensions"))  # for WMTS
    matrix_set = Column(Unicode, label=_(u"Matrix set"))  # for WMTS
    wms_url = Column(Unicode, label=_(u"WMS server URL"))  # for WMTS
    wms_layers = Column(Unicode, label=_(u"WMS layers"))  # for WMTS
    query_layers = Column(Unicode, label=_(u"Query layers"))  # for WMTS
    kml = Column(Unicode, label=_(u"KML 3D"))  # for kml 3D
    is_single_tile = Column(Boolean, label=_(u"Single tile"))  # for extenal WMS
    legend = Column(Boolean, default=True, label=_(u"Display legend"))  # on the tree
    legend_image = Column(Unicode, label=_(u"Legend image"))  # fixed legend image
    legend_rule = Column(Unicode, label=_(u"Legend rule"))  # on wms legend only one rule
    is_legend_expanded = Column(Boolean, default=False, label=_(u"Legend expanded"))
    min_resolution = Column(Float, label=_(u"Min resolution"))  # for all except internal WMS
    max_resolution = Column(Float, label=_(u"Max resolution"))  # for all except internal WMS
    disclaimer = Column(Unicode, label=_(u"Disclaimer"))
    # data attribute field in which application can find a human identifiable name or number
    identifier_attribute_field = Column(Unicode, label=_(u"Identifier attribute field"))
    time_mode = Column(Enum(
        "disabled",
        "single",
        "range",
        native_enum=False), default="disabled", nullable=False,
        label=_(u"Time mode"))
    time_widget = Column(Enum(
        "slider",
        "datepicker",
        native_enum=False), default="slider", nullable=True,
        label=_(u"Time widget"))

    def __init__(
        self, name=u"", public=True, icon=u"",
        layer_type=u"internal WMS"
    ):
        Layer.__init__(self, name=name, public=public)
        self.icon = icon
        self.layer_type = layer_type


class ServerOGC(Base):
    __label__ = _(u"Server OGC")
    __plural__ = _(u"Servers OGC")
    __tablename__ = "server_ogc"
    __table_args__ = {"schema": _schema}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]

    id = Column(
        Integer, primary_key=True
    )
    name = Column(Unicode, label=_(u"Name"))
    description = Column(Unicode, label=_(u"Description"))
    url = Column(Unicode, label=_(u"Base URL"))
    url_wfs = Column(Unicode, label=_(u"WFS URL"))
    type = Column(Unicode, label=_(u"Server type"))
    image_type = Column(Enum(
        "image/jpeg",
        "image/png",
        native_enum=False), label=_(u"Image type"))
    auth = Column(Unicode, label=_(u"Authentication type"))
    wfs_support = Column(Boolean, label=_(u"WFS support"))
    is_single_tile = Column(Boolean, label=_(u"Single tile"))

    def __init__(self, name="", description=None, url=None, url_wfs=None, type=u"mapserver",
                 image_type="image/png", auth=None, wfs_support=False, is_single_tile=False):
        self.name = name
        self.description = description
        self.url = url
        self.url_wfs = url_wfs
        self.type = type
        self.image_type = image_type
        self.auth = auth
        self.wfs_support = wfs_support
        self.is_single_tile = is_single_tile

    def __unicode__(self):
        return self.name or u""  # pragma: nocover


class LayerWMS(Layer):
    __label__ = _(u"WMS layer")
    __plural__ = _(u"WMS layers")
    __tablename__ = "layer_wms"
    __table_args__ = {"schema": _schema}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]
    __mapper_args__ = {"polymorphic_identity": "l_wms"}

    id = Column(
        Integer, ForeignKey(_schema + ".layer.id"), primary_key=True
    )
    server_ogc_id = Column(
        Integer, ForeignKey(_schema + ".server_ogc.id"), nullable=False
    )
    layer = Column(Unicode, label=_(u"WMS layer name"))
    style = Column(Unicode, label=_(u"Style"))
    time_mode = Column(Enum(
        "disabled",
        "single",
        "range",
        native_enum=False), default="disabled", nullable=False,
        label=_(u"Time mode")
    )
    time_widget = Column(Enum(
        "slider",
        "datepicker",
        native_enum=False), default="slider", nullable=True,
        label=_(u"Time widget"))

    # relationship with ServerOGC
    server_ogc = relationship(
        "ServerOGC"
    )

    def __init__(self, name=u"", layer=u"", public=True, icon=u""):
        Layer.__init__(self, name=name, public=public)
        self.layer = layer


class LayerWMTS(Layer):
    __label__ = _(u"WMTS layer")
    __plural__ = _(u"WMTS layers")
    __tablename__ = "layer_wmts"
    __table_args__ = {"schema": _schema}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]
    __mapper_args__ = {"polymorphic_identity": "l_wmts"}

    id = Column(
        Integer, ForeignKey(_schema + ".layer.id"), primary_key=True
    )
    url = Column(Unicode, label=_(u"GetCapabilities URL"))
    layer = Column(Unicode, label=_(u"Layer"))
    style = Column(Unicode, label=_(u"Style"))
    matrix_set = Column(Unicode, label=_(u"Matrix set"))
    image_type = Column(Enum(
        "image/jpeg",
        "image/png",
        native_enum=False), label=_(u"Image type")
    )

    def __init__(self, name=u"", public=True):
        Layer.__init__(self, name=name, public=public)


# association table role <> restriciton area
role_ra = Table(
    "role_restrictionarea", Base.metadata,
    Column(
        "role_id", Integer, ForeignKey(_schema + ".role.id"), primary_key=True
    ),
    Column(
        "restrictionarea_id", Integer,
        ForeignKey(_schema + ".restrictionarea.id"), primary_key=True
    ),
    schema=_schema
)

# association table layer <> restriciton area
layer_ra = Table(
    "layer_restrictionarea", Base.metadata,
    Column(
        "layer_id", Integer,
        ForeignKey(_schema + ".layer.id"), primary_key=True
    ),
    Column(
        "restrictionarea_id", Integer,
        ForeignKey(_schema + ".restrictionarea.id"), primary_key=True
    ),
    schema=_schema
)


class RestrictionArea(Base):
    __label__ = _(u"Restriction Area")
    __plural__ = _(u"Restriction Areas")
    __tablename__ = "restrictionarea"
    __table_args__ = {"schema": _schema}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]

    id = Column(Integer, primary_key=True)
    area = Column(Geometry("POLYGON", srid=_srid, management=management))
    name = Column(Unicode, label=_(u"Name"))
    description = Column(Unicode, label=_(u"Description"))
    readwrite = Column(Boolean, label=_(u"Read-write mode"), default=False)

    # relationship with Role and Layer
    roles = relationship(
        "Role", secondary=role_ra,
        backref="restrictionareas", cascade="save-update,merge,refresh-expire"
    )
    layers = relationship(
        "Layer", secondary=layer_ra,
        backref="restrictionareas", cascade="save-update,merge,refresh-expire"
    )

    def __init__(self, name="", description="", layers=[], roles=[],
                 area=None, readwrite=False):
        self.name = name
        self.description = description
        self.layers = layers
        self.roles = roles
        self.area = area
        self.readwrite = readwrite

    def __unicode__(self):  # pragma: nocover
        return self.name or u""

event.listen(RestrictionArea, "after_insert", cache_invalidate_cb)
event.listen(RestrictionArea, "after_update", cache_invalidate_cb)
event.listen(RestrictionArea, "after_delete", cache_invalidate_cb)


# association table interface <> layer
interface_layer = Table(
    "interface_layer", Base.metadata,
    Column(
        "interface_id", Integer,
        ForeignKey(_schema + ".interface.id"), primary_key=True
    ),
    Column(
        "layer_id", Integer,
        ForeignKey(_schema + ".layer.id"), primary_key=True
    ),
    schema=_schema
)

# association table interface <> theme
interface_theme = Table(
    "interface_theme", Base.metadata,
    Column(
        "interface_id", Integer,
        ForeignKey(_schema + ".interface.id"), primary_key=True
    ),
    Column(
        "theme_id", Integer,
        ForeignKey(_schema + ".theme.id"), primary_key=True
    ),
    schema=_schema
)


class Interface(Base):
    __label__ = _(u"Interface")
    __plural__ = _(u"Interfaces")
    __tablename__ = "interface"
    __table_args__ = {"schema": _schema}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]

    id = Column(Integer, primary_key=True)
    name = Column(Unicode, label=_(u"Name"))
    description = Column(Unicode, label=_(u"Description"))

    # relationship with Layer and Theme
    layers = relationship(
        "Layer", secondary=interface_layer,
        backref="interfaces", cascade="save-update,merge,refresh-expire"
    )
    theme = relationship(
        "Theme", secondary=interface_theme,
        backref="interfaces", cascade="save-update,merge,refresh-expire"
    )

    def __init__(self, name="", description=""):
        self.name = name
        self.description = description

    def __unicode__(self):  # pragma: nocover
        return self.name or u""


class UIMetadata(Base):
    __label__ = _(u"UI metadata")
    __plural__ = _(u"UI metadatas")
    __tablename__ = "ui_metadata"
    __table_args__ = {"schema": _schema}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]

    id = Column(Integer, primary_key=True)
    name = Column(Unicode, label=_(u"Name"))
    value = Column(Unicode, label=_(u"Value"))
    description = Column(Unicode, label=_(u"Description"))

    item_id = Column(
        "item_id", Integer,
        ForeignKey(_schema + ".treeitem.id"), nullable=False
    )
    item = relationship(
        "TreeItem",
        backref=backref(
            "ui_metadatas",
            cascade="save-update,merge,delete,delete-orphan",
        ),
    )

    def __init__(self, name="", value=""):
        self.name = name
        self.value = value

    def __unicode__(self):  # pragma: nocover
        return u"%s: %s" % (self.name or u"", self.value or u"")


class WMTSDimension(Base):
    __label__ = _(u"WMTS dimension")
    __plural__ = _(u"WMTS dimensions")
    __tablename__ = "wmts_dimension"
    __table_args__ = {"schema": _schema}
    __acl__ = [
        (Allow, AUTHORIZED_ROLE, ALL_PERMISSIONS),
    ]

    id = Column(Integer, primary_key=True)
    name = Column(Unicode, label=_(u"Name"))
    value = Column(Unicode, label=_(u"Value"))
    description = Column(Unicode, label=_(u"Description"))

    layer_id = Column(
        "layer_id", Integer,
        ForeignKey(_schema + ".layer_wmts.id"), nullable=False
    )
    layer = relationship(
        "LayerWMTS",
        backref=backref(
            "dimensions",
            cascade="save-update,merge,delete,delete-orphan",
        ),
    )

    def __init__(self, name="", value="", layer=None):
        self.name = name
        self.value = value
        self.layer = layer

    def __unicode__(self):  # pragma: nocover
        return self.name or u""


if _parentschema is not None and _parentschema != "":  # pragma: no cover
    class ParentRole(Base):
        __label__ = _(u"Parent Role")
        __plural__ = _(u"Parent Roles")
        __tablename__ = "role"
        __table_args__ = {"schema": _parentschema}
        __acl__ = [
            (Allow, AUTHORIZED_ROLE, ("view")),
        ]

        id = Column(Integer, primary_key=True)
        name = Column(Unicode, unique=True, nullable=False, label=_(u"Name"))

        def __init__(self, name=u""):
            self.name = name

        def __unicode__(self):
            return self.name or u""  # pragma: nocover


class Shorturl(Base):
    __tablename__ = "shorturl"
    __table_args__ = {"schema": _schema + "_static"}
    __acl__ = [DENY_ALL]
    id = Column(Integer, primary_key=True)
    url = Column(Unicode)
    ref = Column(String(20), index=True, unique=True, nullable=False)
    creator_email = Column(Unicode(200))
    creation = Column(DateTime)
    last_hit = Column(DateTime)
    nb_hits = Column(Integer)



# -*- coding: utf-8 -*-

# Copyright (c) 2014-2016, Camptocamp SA
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The views and conclusions contained in the software and documentation are those
# of the authors and should not be interpreted as representing official policies,
# either expressed or implied, of the FreeBSD Project.

"""create database

Revision ID: 166ff2dcc48d
Revises:
Create Date: 2014-10-24 11:43:23.886123
"""

try:
    from hashlib import sha1
    sha1  # suppress pyflakes warning
except ImportError:  # pragma: nocover
    from sha import new as sha1

from alembic import op, context
from sqlalchemy import ForeignKey, Column, Table, MetaData
from sqlalchemy.types import Integer, Boolean, Unicode, String, Float, \
    UserDefinedType, DateTime

# revision identifiers, used by Alembic.
revision = '166ff2dcc48d'
down_revision = None


class TsVector(UserDefinedType):
    """ A custom type for PostgreSQL's tsvector type. """
    def get_col_spec(self):
        return 'TSVECTOR'


def upgrade():
    schema = context.get_context().config.get_main_option('schema')
    parentschema = context.get_context().config.get_main_option('parentschema')
    srid = context.get_context().config.get_main_option('srid')

    engine = op.get_bind().engine
    if type(engine).__name__ != 'MockConnection' and \
            op.get_context().dialect.has_table(
                engine, 'functionality', schema=schema):  # pragma: nocover
        return

    op.create_table(
        'functionality',
        Column('id', Integer, primary_key=True),
        Column('name', Unicode, nullable=False),
        Column('value', Unicode, nullable=False),
        Column('description', Unicode),
        schema=schema,
    )
    op.create_table(
        'treeitem',
        Column('type', String(10), nullable=False),
        Column('id', Integer, primary_key=True),
        Column('name', Unicode),
        Column('order', Integer, nullable=False),
        Column('metadataURL', Unicode),
        schema=schema,
    )
    op.create_table(
        'restrictionarea',
        Column('id', Integer, primary_key=True),
        Column('name', Unicode),
        Column('description', Unicode),
        Column('readwrite', Boolean, default=False),
        schema=schema,
    )
    op.execute(
        "SELECT AddGeometryColumn('%(schema)s', 'restrictionarea', "
        "'area', %(srid)s, 'POLYGON', 2)" % {
            'schema': schema, 'srid': srid
        }
    )
    op.create_table(
        'shorturl',
        Column('id', Integer, primary_key=True),
        Column('url', Unicode(1000)),
        Column('ref', String(20), index=True, unique=True, nullable=False),
        Column('creator_email', Unicode(200)),
        Column('creation', DateTime),
        Column('last_hit', DateTime),
        Column('nb_hits', Integer),
        schema=schema + '_static',
    )

    op.create_table(
        'role',
        Column('id', Integer, primary_key=True),
        Column('name', Unicode, unique=True, nullable=False),
        Column('description', Unicode),
        schema=schema,
    )
    op.execute(
        "SELECT AddGeometryColumn('%(schema)s', 'role', "
        "'extent', %(srid)s, 'POLYGON', 2)" % {
            'schema': schema, 'srid': srid
        }
    )
    role = Table(
        'role', MetaData(),
        Column('name', Unicode, unique=True, nullable=False),
        schema=schema,
    )
    op.bulk_insert(role, [
        {'name': 'role_admin'}
    ])

    op.create_table(
        'layer',
        Column(
            'id', Integer,
            ForeignKey(schema + '.treeitem.id'), primary_key=True
        ),
        Column('public', Boolean, default=True),
        Column('inMobileViewer', Boolean, default=True),
        Column('inDesktopViewer', Boolean, default=True),
        Column('isChecked', Boolean, default=True),
        Column('icon', Unicode),
        Column('layerType', Unicode(12)),
        Column('url', Unicode),
        Column('imageType', Unicode(10)),
        Column('style', Unicode),
        Column('dimensions', Unicode),
        Column('matrixSet', Unicode),
        Column('wmsUrl', Unicode),
        Column('wmsLayers', Unicode),
        Column('queryLayers', Unicode),
        Column('kml', Unicode),
        Column('isSingleTile', Boolean),
        Column('legend', Boolean, default=True),
        Column('legendImage', Unicode),
        Column('legendRule', Unicode),
        Column('isLegendExpanded', Boolean, default=False),
        Column('minResolution', Float),
        Column('maxResolution', Float),
        Column('disclaimer', Unicode),
        Column('identifierAttributeField', Unicode),
        Column('geoTable', Unicode),
        Column('excludeProperties', Unicode),
        Column('timeMode', Unicode(8)),
        schema=schema,
    )
    op.create_table(
        'role_restrictionarea',
        Column(
            'role_id', Integer,
            ForeignKey(schema + '.role.id'), primary_key=True
        ),
        Column(
            'restrictionarea_id', Integer,
            ForeignKey(schema + '.restrictionarea.id'),
            primary_key=True
        ),
        schema=schema,
    )
    op.create_table(
        'tsearch',
        Column('id', Integer, primary_key=True),
        Column('label', Unicode),
        Column('layer_name', Unicode),
        Column('role_id', Integer, ForeignKey(schema + '.role.id'), nullable=True),
        Column('public', Boolean, server_default='true'),
        Column('ts', TsVector),
        Column('params', Unicode, nullable=True),
        schema=schema,
    )
    op.execute(
        "SELECT AddGeometryColumn('%(schema)s', 'tsearch', 'the_geom', "
        "%(srid)s, 'GEOMETRY', 2)" % {
            'schema': schema, 'srid': srid
        }
    )
    op.create_index(
        'tsearch_ts_idx', 'tsearch', ['ts'],
        schema=schema, postgresql_using='gin'
    )
    op.create_table(
        'treegroup',
        Column(
            'id', Integer,
            ForeignKey(schema + '.treeitem.id'), primary_key=True
        ),
        schema=schema,
    )

    op.create_table(
        'user',
        Column('type', String(10), nullable=False),
        Column('id', Integer, primary_key=True),
        Column('username', Unicode, unique=True, nullable=False),
        Column('password', Unicode, nullable=False),
        Column('email', Unicode, nullable=False),
        Column('is_password_changed', Boolean, default=False),
        Column('role_id', Integer, ForeignKey(schema + '.role.id'), nullable=False),
        schema=schema,
    )
    if parentschema is not None and parentschema is not '':  # pragma: nocover
        op.add_column(
            'user',
            Column('parent_role_id', Integer, ForeignKey(parentschema + '.role.id')),
            schema=schema
        )
    op.execute(
        "INSERT INTO %(schema)s.user (type, username, email, password, role_id) "
        "(SELECT 'user', 'admin', 'info@example.com', '%(pass)s', r.id "
        "FROM %(schema)s.role AS r "
        "WHERE r.name = 'role_admin')" % {
            'schema': schema,
            'pass': sha1('admin').hexdigest()
        }
    )

    op.create_table(
        'role_functionality',
        Column(
            'role_id', Integer,
            ForeignKey(schema + '.role.id'), primary_key=True
        ),
        Column(
            'functionality_id', Integer,
            ForeignKey(schema + '.functionality.id'), primary_key=True
        ),
        schema=schema,
    )
    op.create_table(
        'user_functionality',
        Column(
            'user_id', Integer,
            ForeignKey(schema + '.user.id'), primary_key=True
        ),
        Column(
            'functionality_id', Integer,
            ForeignKey(schema + '.functionality.id'), primary_key=True
        ),
        schema=schema,
    )
    op.create_table(
        'layergroup',
        Column(
            'id', Integer,
            ForeignKey(schema + '.treegroup.id'), primary_key=True
        ),
        Column('isExpanded', Boolean),
        Column('isInternalWMS', Boolean),
        # children have radio button instance of check box
        Column('isBaseLayer', Boolean),
        schema=schema,
    )
    op.create_table(
        'layer_restrictionarea',
        Column(
            'layer_id', Integer,
            ForeignKey(schema + '.layer.id'), primary_key=True
        ),
        Column(
            'restrictionarea_id', Integer,
            ForeignKey(schema + '.restrictionarea.id'), primary_key=True
        ),
        schema=schema,
    )
    op.create_table(
        'layergroup_treeitem',
        Column(
            'treegroup_id', Integer,
            ForeignKey(schema + '.treegroup.id'), primary_key=True
        ),
        Column(
            'treeitem_id', Integer,
            ForeignKey(schema + '.treeitem.id'), primary_key=True
        ),
        schema=schema,
    )
    op.create_table(
        'theme',
        Column(
            'id', Integer,
            ForeignKey(schema + '.treegroup.id'), primary_key=True
        ),
        Column('icon', Unicode),
        Column('inMobileViewer', Boolean, default=False),
        Column('inDesktopViewer', Boolean, default=True),
        schema=schema,
    )
    op.create_table(
        'theme_functionality',
        Column(
            'theme_id', Integer,
            ForeignKey(schema + '.theme.id'), primary_key=True
        ),
        Column(
            'functionality_id', Integer,
            ForeignKey(schema + '.functionality.id'), primary_key=True
        ),
        schema=schema,
    )


def downgrade():
    schema = context.get_context().config.get_main_option('schema')

    op.drop_table('theme_functionality', schema=schema)
    op.drop_table('theme', schema=schema)
    op.drop_table('layergroup_treeitem', schema=schema)
    op.drop_table('layer_restrictionarea', schema=schema)
    op.drop_table('layergroup', schema=schema)
    op.drop_table('user_functionality', schema=schema)
    op.drop_table('role_functionality', schema=schema)
    op.drop_table('user', schema=schema)
    op.drop_table('treegroup', schema=schema)
    op.drop_table('tsearch', schema=schema)
    op.drop_table('role_restrictionarea', schema=schema)
    op.drop_table('layer', schema=schema)
    op.drop_table('role', schema=schema)
    op.drop_table('shorturl', schema=schema + '_static')
    op.drop_table('restrictionarea', schema=schema)
    op.drop_table('treeitem', schema=schema)
    op.drop_table('functionality', schema=schema)
